@@ -137,8 +137,9 @@ start_editing(GtkCellRenderer *renderer, GtkCellEditable *editable, gchar *path,
     if (GTK_IS_SPIN_BUTTON(editable)) {
       gtk_entry_set_alignment(GTK_ENTRY(editable), 1.0);
       gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(editable), FALSE);
-      if (list->max == 36000)
+      if (list->max == 36000) {
 	gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(editable), TRUE);
+      }
     }
     break;
   }
@@ -1498,7 +1499,12 @@ label_sub_window_create(struct SubWin *d, char *title, const char **xpm, const c
   GtkWidget *label;
 
   label = gtk_label_new(NULL);
+#if GTK_CHECK_VERSION(3, 4, 0)
+  gtk_widget_set_halign(label, GTK_ALIGN_START);
+  gtk_widget_set_valign(label, GTK_ALIGN_START);
+#else
   gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.0);
+#endif
   gtk_label_set_selectable(GTK_LABEL(label), TRUE);
   gtk_label_set_line_wrap(GTK_LABEL(label), FALSE);
   gtk_label_set_single_line_mode(GTK_LABEL(label), FALSE);
