@@ -2,7 +2,11 @@
 #define GTK_WIDGET_HEADER
 
 #define SPIN_ENTRY_MAX 1000000
+#if GTK_CHECK_VERSION(3, 4, 0)
+#define NUM_ENTRY_WIDTH 160
+#else
 #define NUM_ENTRY_WIDTH 80
+#endif
 
 enum SPIN_BUTTON_TYPE {
   SPIN_BUTTON_TYPE_WIDTH,
