@@ -680,7 +680,7 @@ set_dir_defs(char *app)
 {
   char *utf8_name, *app_path, *tmp;
 
-  utf8_name = g_locale_from_utf8(app, -1, NULL, NULL, NULL);
+  utf8_name = g_locale_to_utf8(app, -1, NULL, NULL, NULL);
   if (utf8_name == NULL) {
     return 1;
   }
@@ -762,7 +762,9 @@ main(int argc, char **argv)
   g_mem_set_vtable(glib_mem_profiler_table);
 #endif
 
+#if ! GLIB_CHECK_VERSION(2, 32, 0)
   g_thread_init(NULL);
+#endif
   gdk_threads_init();
   gdk_threads_enter();
 
