@@ -292,6 +292,14 @@ static struct menu_config MenuConfigChildGeometry[] = {
   {NULL},
 };
 
+static struct menu_config MenuConfigExtView[] = {
+  {"extwin_dpi",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.exwindpi},
+  {"extwin_width",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.exwinwidth},
+  {"extwin_height",		MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.exwinheight},
+  {"use_external_viewer",	MENU_CONFIG_TYPE_NUMERIC, NULL, &Menulocal.exwin_use_external},
+  {NULL},
+};
+
 static struct menu_config *MenuConfigArrray[] = {
   MenuConfig,
   MenuConfigDriver,
@@ -302,6 +310,7 @@ static struct menu_config *MenuConfigArrray[] = {
   MenuConfigOthers,
   MenuConfigGeometry,
   MenuConfigChildGeometry,
+  MenuConfigExtView,
   NULL,
 };
 
@@ -309,6 +318,7 @@ static NHASH MenuConfigHash = NULL;
 
 #define BUF_SIZE 64
 
+#if ! GTK_CHECK_VERSION(3, 4, 0)
 static void
 get_palette(void)
 {
@@ -359,6 +369,7 @@ set_palette(void)
 
   return;
 }
+#endif
 
 static void
 add_str_with_int_to_array(struct menu_config *cfg, struct narray *conf)
@@ -568,12 +579,18 @@ menu_save_config(int type)
     menu_save_config_sub(MenuConfigViewer, &conf);
   }
 
+  if (type & SAVE_CONFIG_TYPE_EXTERNAL_VIEWER) {
+    menu_save_config_sub(MenuConfigExtView, &conf);
+  }
+
   if (type & SAVE_CONFIG_TYPE_TOGGLE_VIEW) {
     menu_save_config_sub(MenuConfigToggleView, &conf);
   }
 
   if (type & SAVE_CONFIG_TYPE_OTHERS) {
+#if ! GTK_CHECK_VERSION(3, 4, 0)
     get_palette();
+#endif
     menu_save_config_sub(MenuConfigOthers, &conf);
   }
 
@@ -974,56 +991,6 @@ mgtkwindowconfig(void)
   return 0;
 }
 
-static int
-exwinloadconfig(void)
-{
-  FILE *fp;
-  char *tok, *str, *s2;
-  char *f1;
-  int val;
-  char *endptr;
-  int len;
-
-  fp = openconfig(G2WINCONF);
-  if (fp == NULL)
-    return 0;
-
-  while ((tok = getconfig(fp, &str))) {
-    s2 = str;
-    if (strcmp(tok, "win_dpi") == 0) {
-      f1 = getitok2(&s2, &len, " \t,");
-      val = strtol(f1, &endptr, 10);
-      if (endptr[0] == '\0')
-	Menulocal.exwindpi = val;
-      g_free(f1);
-    } else if (strcmp(tok, "win_width") == 0) {
-      f1 = getitok2(&s2, &len, " \t,");
-      val = strtol(f1, &endptr, 10);
-      if (endptr[0] == '\0')
-	Menulocal.exwinwidth = val;
-      g_free(f1);
-    } else if (strcmp(tok, "win_height") == 0) {
-      f1 = getitok2(&s2, &len, " \t,");
-      val = strtol(f1, &endptr, 10);
-      if (endptr[0] == '\0')
-	Menulocal.exwinheight = val;
-      g_free(f1);
-    } else if (strcmp(tok, "use_external_viewer") == 0) {
-      f1 = getitok2(&s2, &len, " \t,");
-      val = strtol(f1, &endptr, 10);
-      if (endptr[0] == '\0')
-	Menulocal.exwin_use_external = val;
-      g_free(f1);
-    } else {
-      fprintf(stderr, "(%s): configuration '%s' in section %s is not used.\n", AppName, tok, G2WINCONF);
-    }
-    g_free(tok);
-    g_free(str);
-  }
-  closeconfig(fp);
-  return 0;
-}
-
 void
 menuadddrawrable(struct objlist *parent, struct narray *drawrable)
 {
@@ -1041,12 +1008,28 @@ menuadddrawrable(struct objlist *parent, struct narray *drawrable)
   }
 }
 
+static void
+free_script_list(struct script *script)
+{
+  struct script *scur, *sdel;
+
+  scur = script;
+  while (scur) {
+    sdel = scur;
+    scur = scur->next;
+    g_free(sdel->name);
+    g_free(sdel->script);
+    g_free(sdel->description);
+    g_free(sdel->option);
+    g_free(sdel);
+  }
+}
+
 static void
 menulocal_finalize(void)
 {
   struct extprinter *pcur, *pdel;
   struct character_map_list *cmap, *cmap_tmp;
-  struct script *scur, *sdel;
   int i, j;
   struct menu_config *cfg;
 
@@ -1081,18 +1064,12 @@ menulocal_finalize(void)
   }
   Menulocal.extprinterroot = NULL;
 
-  scur = Menulocal.scriptroot;
-  while (scur) {
-    sdel = scur;
-    scur = scur->next;
-    g_free(sdel->name);
-    g_free(sdel->script);
-    g_free(sdel->description);
-    g_free(sdel->option);
-    g_free(sdel);
-  }
+  free_script_list(Menulocal.scriptroot);
   Menulocal.scriptroot = NULL;
 
+  free_script_list(Menulocal.addin_list);
+  Menulocal.addin_list = NULL;
+
   if (Menulocal.pix) {
     cairo_surface_destroy(Menulocal.pix);
   }
@@ -1144,7 +1121,10 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   Menulocal.changedirectory = 1;
   set_paper_type(21000, 29700);
   Menulocal.PaperZoom = 10000;
-  Menulocal.exwindpi = DEFAULT_DPI;
+  Menulocal.exwindpi = DEFAULT_DPI / 2;
+  Menulocal.exwinwidth = 400;
+  Menulocal.exwinheight = 600;
+  Menulocal.exwin_use_external = TRUE;
   Menulocal.expand = 1;
   Menulocal.expanddir = g_strdup("./");
   Menulocal.expandtofullpath = TRUE;
@@ -1178,15 +1158,12 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   if (mgtkloadconfig())
     goto errexit;
 
-  if (exwinloadconfig())
-    goto errexit;
-
   gra2cairo_set_antialias(Menulocal.local, Menulocal.antialias);
   if (_putobj(obj, "antialias", inst, &(Menulocal.antialias)))
     goto errexit;
 
   if (Menulocal.exwindpi < 1)
-    Menulocal.exwindpi = DEFAULT_DPI;
+    Menulocal.exwindpi = DEFAULT_DPI / 2;
 
   if (Menulocal.exwindpi > DPI_MAX)
     Menulocal.exwindpi = DPI_MAX;
@@ -1222,7 +1199,9 @@ menuinit(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **arg
   Menulocal.pix = NULL;
   Menulocal.lock = 0;
 
+#if ! GTK_CHECK_VERSION(3, 4, 0)
   set_palette();
+#endif
 
   if (!OpenApplication()) {
     error(obj, ERR_MENU_DISPLAY);
@@ -1495,7 +1474,7 @@ mxdpi(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
     dpi = DPI_MAX;
   Menulocal.windpi = dpi;
   Menulocal.local->pixel_dot_x =
-      Menulocal.local->pixel_dot_y =dpi / (DPI_MAX * 1.0);
+    Menulocal.local->pixel_dot_y = dpi / (DPI_MAX * 1.0);
   *(int *) argv[2] = dpi;
 
   if (NgraphApp.Viewer.gdk_win) {
@@ -1540,6 +1519,7 @@ mx_clear(GdkRegion *region)
     gdk_cairo_region(cr, region);
     cairo_fill(cr);
   } else {
+    cairo_reset_clip(cr);
     cairo_paint(cr);
   }
 
@@ -2037,6 +2017,116 @@ mx_get_locale(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char
   return 0;
 }
 
+static int
+mx_addin_list_append(struct objlist *obj, N_VALUE *inst, N_VALUE *rval, int argc, char **argv)
+{
+  int n, id, i;
+  char *sarray, *name, *script, *description, *option, *argv2[2];
+  struct objlist *sa_obj;
+  struct narray iarray;
+  struct script *addin, *list;
+
+  sarray = (char *) argv[2];
+  if (sarray  == NULL) {
+    return 0;
+  }
+
+  arrayinit(&iarray, sizeof(int));
+  if (getobjilist(sarray, &sa_obj, &iarray, FALSE, NULL)) {
+    return 0;
+  }
+
+  if (g_strcmp0(chkobjectname(sa_obj), "sarray") != 0) {
+    return 1;
+  }
+
+  n = arraynum(&iarray);
+  if (n < 1) {
+    arraydel(&iarray);
+    return 0;
+  }
+
+  id = arraynget_int(&iarray, 0);
+
+  if (getobj(sa_obj, "num", id, 0, NULL, &n) < 0) {
+    return 0;
+  }
+
+  if (n < 4) {
+    arraydel(&iarray);
+    return 0;
+  }
+
+  addin = g_malloc0(sizeof(*addin));
+  if (addin == NULL) {
+    return 1;
+  }
+
+  argv2[0] = (char *) &i;
+  argv2[1] = NULL;
+
+  i = 0;
+  getobj(sa_obj, "get", id, 1, argv2, &script);
+  if (! g_utf8_validate(script, -1, NULL)) {
+    goto Err;
+  }
+  addin->script = g_strdup(script);
+
+  i = 1;
+  getobj(sa_obj, "get", id, 1, argv2, &name);
+  if (! g_utf8_validate(name, -1, NULL)) {
+    goto Err;
+  }
+  addin->name = g_strdup(name);
+
+  i = 2;
+  getobj(sa_obj, "get", id, 1, argv2, &description);
+  if (! g_utf8_validate(description, -1, NULL)) {
+    goto Err;
+  }
+  addin->description = g_strdup(description);
+
+  i = 3;
+  getobj(sa_obj, "get", id, 1, argv2, &option);
+  if (! g_utf8_validate(option, -1, NULL)) {
+    goto Err;
+  }
+  addin->option = g_strdup(option);
+
+  addin->next = NULL;
+
+  if (Menulocal.addin_list == NULL) {
+    Menulocal.addin_list = addin;
+  } else {
+    for (list = Menulocal.addin_list; list != addin; list = list->next) {
+      if (list->next == NULL) {
+	list->next = addin;
+      }
+    }
+  }
+
+  arraydel(&iarray);
+
+  return 0;
+
+ Err:
+  if (addin->script) {
+    g_free(addin->script);
+  }
+  if (addin->name) {
+    g_free(addin->name);
+  }
+  if (addin->description) {
+    g_free(addin->description);
+  }
+  if (addin->option) {
+    g_free(addin->option);
+  }
+  g_free(addin);
+
+  return 0;
+}
+
 int
 get_graph_modified(void)
 {
@@ -2103,6 +2193,7 @@ static struct objtable gtkmenu[] = {
   {"focus", NVFUNC, NREAD | NEXEC, mx_focus_obj, "o", 0},
   {"unfocus", NVFUNC, NREAD | NEXEC, mx_unfocus_obj, "", 0},
   {"locale", NVFUNC, NREAD | NEXEC, mx_get_locale, "", 0},
+  {"addin_list_append", NVFUNC, NREAD | NEXEC, mx_addin_list_append, "o", 0},
   {"_evloop", NVFUNC, 0, mx_evloop, NULL, 0},
 };
 
