@@ -118,7 +118,7 @@ struct menulocal
   int PaperZoom;
   struct narray drawrable;
   struct extprinter *extprinterroot;
-  struct script *scriptroot;
+  struct script *scriptroot, *addin_list;
   int menuheight, menuwidth, menux, menuy;
   int fileheight, filewidth, filex, filey, fileopen;
   int axisheight, axiswidth, axisx, axisy, axisopen;
@@ -161,6 +161,7 @@ enum SAVE_CONFIG_TYPE {
 #define SAVE_CONFIG_TYPE_X11MENU 	(SAVE_CONFIG_TYPE_GEOMETRY		\
 				  | SAVE_CONFIG_TYPE_CHILD_GEOMETRY	\
 				  | SAVE_CONFIG_TYPE_VIEWER		\
+				  | SAVE_CONFIG_TYPE_EXTERNAL_VIEWER	\
 				  | SAVE_CONFIG_TYPE_EXTERNAL_DRIVER	\
 				  | SAVE_CONFIG_TYPE_ADDIN_SCRIPT	\
 				  | SAVE_CONFIG_TYPE_MISC)
