@@ -113,11 +113,6 @@ static struct subwin_popup_list Popup_list[] = {
 #define ID_BUF_SIZE 16
 #define TITLE_BUF_SIZE 128
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-static gboolean AxisWinExpose(GtkWidget *wi, cairo_t *cr, gpointer client_data);
-#else
-static gboolean AxisWinExpose(GtkWidget *wi, GdkEvent *event, gpointer client_data);
-#endif
 static void axis_list_set_val(struct SubWin *d, GtkTreeIter *iter, int row);
 static int check_axis_history(struct objlist *obj);
 
@@ -349,9 +344,17 @@ GridDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
   d = (struct GridDialog *) data;
   if (makewidget) {
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
+#endif
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     j = 0;
     w = combo_box_entry_create();
@@ -377,7 +380,11 @@ GridDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     j = 0;
     w = create_color_button(wi);
@@ -400,7 +407,12 @@ GridDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(d->vbox), hbox, FALSE, FALSE, 4);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 4, FALSE);
+#endif
+
     j = 0;
     for (i = 0; i < GRID_DIALOG_STYLE_NUM; i++) {
       gauge_syle_setup(d, table, i, j++);
@@ -611,9 +623,17 @@ SectionDialogSetup(GtkWidget *wi, void *data, int makewidget)
   d = (struct SectionDialog *) data;
 
   if (makewidget) {
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
+#endif
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_POSITION, TRUE, TRUE);
@@ -627,7 +647,11 @@ SectionDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 4);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_POSITION, TRUE, TRUE);
@@ -642,9 +666,15 @@ SectionDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(d->vbox), hbox, FALSE, FALSE, 4);
 
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
 
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     w = gtk_button_new_with_mnemonic(_("_X axis"));
     g_signal_connect(w, "clicked", G_CALLBACK(SectionDialogAxisX), d);
@@ -658,7 +688,11 @@ SectionDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 4);
 
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     w = gtk_button_new_with_mnemonic(_("_Y axis"));
     g_signal_connect(w, "clicked", G_CALLBACK(SectionDialogAxisY), d);
@@ -672,7 +706,11 @@ SectionDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 4);
 
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     w = gtk_button_new_with_mnemonic(_("_U axis"));
     g_signal_connect(w, "clicked", G_CALLBACK(SectionDialogAxisU), d);
@@ -686,7 +724,11 @@ SectionDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 4);
 
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     w = gtk_button_new_with_mnemonic(_("_R axis"));
     g_signal_connect(w, "clicked", G_CALLBACK(SectionDialogAxisR), d);
@@ -700,7 +742,11 @@ SectionDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 4);
 
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     w = gtk_button_new_with_mnemonic(_("_Grid"));
     g_signal_connect(w, "clicked", G_CALLBACK(SectionDialogGrid), d);
@@ -835,9 +881,17 @@ CrossDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
   d = (struct CrossDialog *) data;
   if (makewidget) {
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
+#endif
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_POSITION, TRUE, TRUE);
@@ -851,7 +905,11 @@ CrossDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox), table, TRUE, TRUE, 4);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_POSITION, TRUE, TRUE);
@@ -867,8 +925,13 @@ CrossDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(d->vbox), hbox, FALSE, FALSE, 4);
 
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     w = gtk_button_new_with_mnemonic(_("_X axis"));
     g_signal_connect(w, "clicked", G_CALLBACK(CrossDialogAxisX), d);
@@ -882,7 +945,11 @@ CrossDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 4);
 
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     w = gtk_button_new_with_mnemonic(_("_Y axis"));
     g_signal_connect(w, "clicked", G_CALLBACK(CrossDialogAxisY), d);
@@ -978,7 +1045,11 @@ ZoomDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
   d = (struct ZoomDialog *) data;
   if (makewidget) {
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_PERCENT, TRUE, TRUE);
     item_setup(vbox, w, _("_Zoom:"), TRUE);
     d->zoom_entry = w;
@@ -1247,7 +1318,11 @@ scale_tab_create(struct AxisDialog *d)
   GtkWidget *parent_box, *w, *frame, *table, *hbox;
   int i;
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+  table = gtk_grid_new();
+#else
   table = gtk_table_new(1, 2, FALSE);
+#endif
 
   i = 0;
   w = combo_box_entry_create();
@@ -1263,7 +1338,11 @@ scale_tab_create(struct AxisDialog *d)
   d->inc = w;
 
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 12);
+#else
   hbox = gtk_hbox_new(FALSE, 12);
+#endif
 
   w = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
   g_signal_connect(w, "clicked", G_CALLBACK(AxisDialogClear), d);
@@ -1297,7 +1376,11 @@ scale_tab_create(struct AxisDialog *d)
   frame = gtk_frame_new(_("Scale"));
   gtk_container_add(GTK_CONTAINER(frame), table);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  parent_box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
   parent_box = gtk_vbox_new(FALSE, 4);
+#endif
   gtk_box_pack_start(GTK_BOX(parent_box), frame, TRUE, TRUE, 4);
 
   add_copy_button_to_box(parent_box, G_CALLBACK(scale_tab_copy_clicked), d, "axis");
@@ -1396,9 +1479,17 @@ baseline_tab_create(GtkWidget *wi, struct AxisDialog *dd)
 
   d = &dd->base;
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
   hbox = gtk_hbox_new(FALSE, 4);
+#endif
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+  table = gtk_grid_new();
+#else
   table = gtk_table_new(1, 2, FALSE);
+#endif
 
   i = 0;
   w = gtk_check_button_new_with_mnemonic(_("Draw _Baseline"));
@@ -1423,8 +1514,16 @@ baseline_tab_create(GtkWidget *wi, struct AxisDialog *dd)
   gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+  table = gtk_grid_new();
+#else
   table = gtk_table_new(1, 2, FALSE);
+#endif
+#if GTK_CHECK_VERSION(3, 0, 0)
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
   vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
   i = 0;
   w = combo_box_create();
@@ -1446,7 +1545,11 @@ baseline_tab_create(GtkWidget *wi, struct AxisDialog *dd)
   gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+  table = gtk_grid_new();
+#else
   table = gtk_table_new(1, 2, FALSE);
+#endif
 
   i = 0;
   w = combo_box_create();
@@ -1458,7 +1561,6 @@ baseline_tab_create(GtkWidget *wi, struct AxisDialog *dd)
   d->wavelen = w;
 
   w = create_spin_entry_type(SPIN_BUTTON_TYPE_WIDTH, TRUE, TRUE);
-  gtk_widget_set_size_request(w, NUM_ENTRY_WIDTH, -1);
   add_widget_to_table(table, w, _("_Wave width:"), FALSE, i++);
   d->wavewid = w;
 
@@ -1468,7 +1570,11 @@ baseline_tab_create(GtkWidget *wi, struct AxisDialog *dd)
 
   gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
   vbox = gtk_vbox_new(FALSE, 4);
+#endif
   gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 4);
 
   add_copy_button_to_box(vbox, G_CALLBACK(baseline_tab_copy_clicked), dd, "axis");
@@ -1567,9 +1673,17 @@ gauge_tab_create(GtkWidget *wi, struct AxisDialog *dd)
 
   d = &dd->gauge;
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
   vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+  table = gtk_grid_new();
+#else
   table = gtk_table_new(1, 2, FALSE);
+#endif
 
   j = 0;
   w = combo_box_create();
@@ -1591,7 +1705,11 @@ gauge_tab_create(GtkWidget *wi, struct AxisDialog *dd)
   gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+  table = gtk_grid_new();
+#else
   table = gtk_table_new(1, 4, FALSE);
+#endif
 
   j = 0;
   w = combo_box_entry_create();
@@ -1618,7 +1736,11 @@ gauge_tab_create(GtkWidget *wi, struct AxisDialog *dd)
   gtk_container_add(GTK_CONTAINER(frame), table);
   gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  parent_box = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
   parent_box = gtk_vbox_new(FALSE, 4);
+#endif
   gtk_box_pack_start(GTK_BOX(parent_box), vbox, TRUE, TRUE, 4);
   add_copy_button_to_box(parent_box, G_CALLBACK(gauge_tab_copy_clicked), dd, "axis");
 
@@ -1819,12 +1941,23 @@ numbering_tab_create(GtkWidget *wi, struct AxisDialog *dd)
 
   d = &dd->numbering;
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+#else
   hbox = gtk_hbox_new(FALSE, 4);
 
   vbox = gtk_vbox_new(FALSE, 0);
+#endif
 
 
+
+#if GTK_CHECK_VERSION(3, 4, 0)
+  table = gtk_grid_new();
+#else
   table = gtk_table_new(1, 2, FALSE);
+#endif
 
   i = 0;
   w = combo_box_create();
@@ -1848,7 +1981,11 @@ numbering_tab_create(GtkWidget *wi, struct AxisDialog *dd)
   gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+  table = gtk_grid_new();
+#else
   table = gtk_table_new(1, 2, FALSE);
+#endif
 
   i = 0;
   w = combo_box_create();
@@ -1875,7 +2012,11 @@ numbering_tab_create(GtkWidget *wi, struct AxisDialog *dd)
   gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+  table = gtk_grid_new();
+#else
   table = gtk_table_new(1, 2, FALSE);
+#endif
 
   i = 0;
   w = combo_box_create();
@@ -1916,7 +2057,11 @@ numbering_tab_create(GtkWidget *wi, struct AxisDialog *dd)
 
   gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
   vbox = gtk_vbox_new(FALSE, 4);
+#endif
   gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 4);
 
   add_copy_button_to_box(vbox, G_CALLBACK(numbering_tab_copy_clicked), dd, "axis");
@@ -2017,7 +2162,11 @@ font_tab_create(GtkWidget *wi, struct AxisDialog *dd)
 
   d = &dd->font;
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+  table = gtk_grid_new();
+#else
   table = gtk_table_new(1, 2, FALSE);
+#endif
 
   i = 0;
   w = create_spin_entry_type(SPIN_BUTTON_TYPE_POINT, TRUE, TRUE);
@@ -2036,7 +2185,11 @@ font_tab_create(GtkWidget *wi, struct AxisDialog *dd)
   add_widget_to_table(table, w, _("_Font:"), FALSE, i++);
   d->font = w;
 
+#if GTK_CHECK_VERSION(3, 2, 0)
+  btn_box = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
+#else
   btn_box = gtk_hbutton_box_new();
+#endif
   gtk_box_set_spacing(GTK_BOX(btn_box), 10);
   w = gtk_check_button_new_with_label("gtk-bold");
   gtk_button_set_use_stock(GTK_BUTTON(w), TRUE);
@@ -2057,7 +2210,11 @@ font_tab_create(GtkWidget *wi, struct AxisDialog *dd)
   frame = gtk_frame_new(_("Font"));
   gtk_container_add(GTK_CONTAINER(frame), table);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
   vbox = gtk_vbox_new(FALSE, 4);
+#endif
   gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 4);
 
   add_copy_button_to_box(vbox, G_CALLBACK(font_tab_copy_clicked), dd, "axis");
@@ -2155,7 +2312,11 @@ position_tab_create(GtkWidget *wi, struct AxisDialog *dd)
 
   d = &dd->position;
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+  table = gtk_grid_new();
+#else
   table = gtk_table_new(1, 2, FALSE);
+#endif
 
   i = 0;
   w = create_spin_entry_type(SPIN_BUTTON_TYPE_POSITION, TRUE, TRUE);
@@ -2175,7 +2336,11 @@ position_tab_create(GtkWidget *wi, struct AxisDialog *dd)
   d->direction = w;
 
   w = combo_box_entry_create();
+#if GTK_CHECK_VERSION(3, 4, 0)
+  gtk_widget_set_size_request(w, NUM_ENTRY_WIDTH, -1);
+#else
   gtk_widget_set_size_request(w, NUM_ENTRY_WIDTH * 2, -1);
+#endif
   g_signal_connect(w, "changed", G_CALLBACK(AxisDialogRef), dd);
   add_widget_to_table(table, w, _("_Adjust:"), FALSE, i++);
   d->adjust = w;
@@ -2188,7 +2353,11 @@ position_tab_create(GtkWidget *wi, struct AxisDialog *dd)
   frame = gtk_frame_new(_("Position"));
   gtk_container_add(GTK_CONTAINER(frame), table);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
   vbox = gtk_vbox_new(FALSE, 4);
+#endif
   gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 4);
 
   add_copy_button_to_box(vbox, G_CALLBACK(position_tab_copy_clicked), dd, "axis");
@@ -2773,9 +2942,18 @@ AxisWinUpdate(int clear)
 {
   struct SubWin *d;
 
-  d = &(NgraphApp.AxisWin);
+  if (Menulock || Globallock)
+    return;
 
-  AxisWinExpose(NULL, NULL, NULL);
+  d = &(NgraphApp.AxisWin);
+  if (d->text == NULL)
+    return;
+
+  if (list_sub_window_must_rebuild(d)) {
+    list_sub_window_build(d, axis_list_set_val);
+  } else {
+    list_sub_window_set(d, axis_list_set_val);
+  }
 
   if (! clear && d->select >= 0) {
     list_store_select_int(GTK_WIDGET(d->text), AXIS_WIN_COL_ID, d->select);
@@ -2847,32 +3025,6 @@ axis_list_set_val(struct SubWin *d, GtkTreeIter *iter, int row)
   }
 }
 
-static gboolean
-#if GTK_CHECK_VERSION(3, 0, 0)
-AxisWinExpose(GtkWidget *wi, cairo_t *cr, gpointer client_data)
-#else
-AxisWinExpose(GtkWidget *wi, GdkEvent *event, gpointer client_data)
-#endif
-{
-  struct SubWin *d;
-
-  if (Menulock || Globallock)
-    return FALSE;
-
-  d = &(NgraphApp.AxisWin);
-
-  if (d->text == NULL)
-    return FALSE;
-
-  if (list_sub_window_must_rebuild(d)) {
-    list_sub_window_build(d, axis_list_set_val);
-  } else {
-    list_sub_window_set(d, axis_list_set_val);
-  }
-
-  return FALSE;
-}
-
 static int
 check_axis_history(struct objlist *obj)
 {
@@ -3327,7 +3479,6 @@ CmAxisWindow(GtkToggleAction *action, gpointer client_data)
 {
   struct SubWin *d;
   int state;
-  GtkWidget *dlg;
 
   d = &(NgraphApp.AxisWin);
 
@@ -3352,13 +3503,7 @@ CmAxisWindow(GtkToggleAction *action, gpointer client_data)
   d->ev_key = axiswin_ev_key_down;
   d->delete = AxisDel;
 
-  dlg = list_sub_window_create(d, "Axis Window", AXIS_WIN_COL_NUM, Alist, Axiswin_xpm, Axiswin48_xpm);
-
-#if GTK_CHECK_VERSION(3, 0, 0)
-  g_signal_connect(dlg, "draw", G_CALLBACK(AxisWinExpose), NULL);
-#else
-  g_signal_connect(dlg, "expose-event", G_CALLBACK(AxisWinExpose), NULL);
-#endif
+  list_sub_window_create(d, "Axis Window", AXIS_WIN_COL_NUM, Alist, Axiswin_xpm, Axiswin48_xpm);
 
   d->obj = chkobject("axis");
   d->num = chkobjlastinst(d->obj);
@@ -3381,4 +3526,6 @@ CmAxisWindow(GtkToggleAction *action, gpointer client_data)
 
   sub_window_show_all(d);
   sub_window_set_geometry(d, TRUE);
+
+  AxisWinUpdate(TRUE);
 }
