@@ -161,7 +161,11 @@ PageDialogSetup(GtkWidget *wi, void *data, int makewidget)
   d = (struct PageDialog *) data;
 
   if (makewidget) {
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_LENGTH, FALSE, TRUE);
@@ -659,9 +663,15 @@ SwitchDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
   if (makewidget) {
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
 
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     label = gtk_label_new_with_mnemonic(_("_Draw order"));
     gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 4);
@@ -682,7 +692,11 @@ SwitchDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 4);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     w = gtk_button_new_from_stock(GTK_STOCK_ADD);
     g_signal_connect(w, "clicked", G_CALLBACK(SwitchDialogAdd), d);
@@ -694,7 +708,11 @@ SwitchDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 4);
     d->ins = w;
 
+#if GTK_CHECK_VERSION(3, 2, 0)
+    w = gtk_separator_new(GTK_ORIENTATION_HORIZONTAL);
+#else
     w = gtk_hseparator_new();
+#endif
     gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 4);
 
     w = gtk_button_new_from_stock(GTK_STOCK_GOTO_TOP);
@@ -722,12 +740,20 @@ SwitchDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 4);
     d->del = w;
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox2 = gtk_vbox_new(FALSE, 4);
+#endif
     gtk_box_pack_end(GTK_BOX(vbox2), vbox, FALSE, FALSE, 4);
 
     gtk_box_pack_start(GTK_BOX(hbox), vbox2, FALSE, FALSE, 4);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     label = gtk_label_new_with_mnemonic(_("_Objects"));
     gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 4);
@@ -828,21 +854,38 @@ DirectoryDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
   d = (struct DirectoryDialog *) data;
   if (makewidget) {
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(2, 2, FALSE);
+#endif
 
     w = gtk_file_chooser_button_new(_("directory"), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER);
     d->dir = w;
     add_widget_to_table(table, w, _("_Select Dir:"), TRUE, 0);
 
     w = gtk_label_new(_("Current Dir:"));
+#if GTK_CHECK_VERSION(3, 4, 0)
+    gtk_widget_set_halign(w, GTK_ALIGN_START);
+    g_object_set(w, "margin", GINT_TO_POINTER(4), NULL);
+    gtk_grid_attach(GTK_GRID(table), w, 0, 1, 1, 1);
+#else
     gtk_misc_set_alignment(GTK_MISC(w), 0, 0.5);
     gtk_table_attach(GTK_TABLE(table), w, 0, 1, 1, 2, 0, 0, 4, 4);
+#endif
 
     w = gtk_label_new("");
     gtk_label_set_ellipsize(GTK_LABEL(w), PANGO_ELLIPSIZE_START);
-    gtk_misc_set_alignment(GTK_MISC(w), 0, 0.5);
     d->dir_label = w;
+#if GTK_CHECK_VERSION(3, 4, 0)
+    gtk_widget_set_hexpand(w, TRUE);
+    gtk_widget_set_halign(w, GTK_ALIGN_START);
+    g_object_set(w, "margin", GINT_TO_POINTER(4), NULL);
+    gtk_grid_attach(GTK_GRID(table), w, 1, 2, 1, 1);
+#else
+    gtk_misc_set_alignment(GTK_MISC(w), 0, 0.5);
     gtk_table_attach(GTK_TABLE(table), w, 1, 2, 1, 2, GTK_FILL | GTK_EXPAND, 0, 4, 4);
+#endif
 
     gtk_box_pack_start(GTK_BOX(d->vbox), table, FALSE, FALSE, 4);
   }
@@ -892,7 +935,11 @@ LoadDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
   d = (struct LoadDialog *) data;
   if (makewidget) {
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     w = gtk_check_button_new_with_mnemonic(_("_Expand included file"));
     d->expand_file = w;
@@ -948,7 +995,11 @@ PrmDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
   d = (struct PrmDialog *) data;
   if (makewidget) {
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
     w = gtk_check_button_new_with_mnemonic(_("_Ignore file path"));
     d->ignore_path = w;
     gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 4);
@@ -994,7 +1045,11 @@ SaveDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
   d = (struct SaveDialog *) data;
   if (makewidget) {
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     w = combo_box_create();
     item_setup(vbox, w, _("_Path:"), FALSE);
