@@ -101,12 +101,9 @@ static struct subwin_popup_list Popup_list[] = {
 #define POPUP_ITEM_DOWN 9
 #define POPUP_ITEM_BOTTOM 10
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-static gboolean LegendWinExpose(GtkWidget *wi, cairo_t *cr, gpointer client_data);
-#else
-static gboolean LegendWinExpose(GtkWidget *wi, GdkEvent *event, gpointer client_data);
-#endif
 static void LegendDialogCopy(struct LegendDialog *d);
+static void legend_list_build(struct LegendWin *d);
+static void legend_list_set(struct LegendWin *d);
 
 static char *legendlist[LEGENDNUM] = {
   N_("path"),
@@ -866,12 +863,21 @@ points_setup(struct LegendDialog *d)
 					    renderer_func,
 					    GINT_TO_POINTER(i),
 					    NULL);
+    gtk_tree_view_column_set_expand(col, TRUE);
   }
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
   vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
   label = gtk_label_new_with_mnemonic(_("_Points:"));
+#if GTK_CHECK_VERSION(3, 4, 0)
+  gtk_widget_set_halign(label, GTK_ALIGN_START);
+#else
   gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
+#endif
   gtk_label_set_mnemonic_widget(GTK_LABEL(label), tree_view);
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 2);
 
@@ -882,7 +888,11 @@ points_setup(struct LegendDialog *d)
   gtk_container_add(GTK_CONTAINER(swin), tree_view);
   gtk_box_pack_start(GTK_BOX(vbox), swin, TRUE, TRUE, 2);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+  hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
   hbox = gtk_hbox_new(FALSE, 4);
+#endif
 
   btn = gtk_button_new_from_stock(GTK_STOCK_ADD);
   g_signal_connect(btn, "clicked", G_CALLBACK(insert_column), tree_view);
@@ -1099,6 +1109,18 @@ LegendArrowDialogScaleL(GtkWidget *w, gpointer client_data)
 #endif
 }
 
+static gchar*
+format_value_percent(GtkScale *scale, gdouble value, gpointer user_data)
+{
+  return g_strdup_printf ("%.0f%%", value);
+}
+
+static gchar*
+format_value_degree(GtkScale *scale, gdouble value, gpointer user_data)
+{
+  return g_strdup_printf ("%.0f°", value);
+}
+
 static void
 LegendArrowDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
@@ -1116,18 +1138,39 @@ LegendArrowDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     gtk_dialog_add_button(GTK_DIALOG(wi), GTK_STOCK_DELETE, IDDELETE);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
+#endif
 
     w = points_setup(d);
 
     frame = gtk_frame_new(NULL);
     gtk_container_add(GTK_CONTAINER(frame), w);
+#if GTK_CHECK_VERSION(3, 4, 0)
+   gtk_box_pack_start(GTK_BOX(hbox), frame, TRUE, TRUE, 0);
+#else
     gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
+#endif
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+#else
     vbox2 = gtk_vbox_new(FALSE, 0);
+#endif
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox2 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
     hbox2 = gtk_hbox_new(FALSE, 4);
+#endif
+
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = combo_box_create();
@@ -1142,7 +1185,11 @@ LegendArrowDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 0);
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = gtk_check_button_new_with_mnemonic(_("_Close path"));
@@ -1162,9 +1209,18 @@ LegendArrowDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     gtk_box_pack_start(GTK_BOX(hbox2), table, TRUE, TRUE, 0);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
+#if GTK_CHECK_VERSION(3, 2, 0)
+    w = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, 10, 170, 1);
+#else
     w = gtk_hscale_new_with_range(10, 170, 1);
+#endif
     g_signal_connect(w, "value-changed", G_CALLBACK(LegendArrowDialogScaleL), d);
+    g_signal_connect(w, "format-value", G_CALLBACK(format_value_degree), NULL);
     gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 4);
     d->arrow_length = w;
 
@@ -1178,8 +1234,13 @@ LegendArrowDialogSetup(GtkWidget *wi, void *data, int makewidget)
 #endif
     d->view = w;
 
+#if GTK_CHECK_VERSION(3, 2, 0)
+    w = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, 100, 2000, 1);
+#else
     w = gtk_hscale_new_with_range(100, 2000, 1);
+#endif
     g_signal_connect(w, "value-changed", G_CALLBACK(LegendArrowDialogScaleW), d);
+    g_signal_connect(w, "format-value", G_CALLBACK(format_value_percent), NULL);
     gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 4);
     d->arrow_width = w;
 
@@ -1194,7 +1255,11 @@ LegendArrowDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_container_add(GTK_CONTAINER(frame), hbox2);
     gtk_box_pack_start(GTK_BOX(vbox2), frame, FALSE, FALSE, 0);
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = combo_box_create();
@@ -1250,9 +1315,17 @@ LegendRectDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     gtk_dialog_add_button(GTK_DIALOG(wi), GTK_STOCK_DELETE, IDDELETE);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
+#endif
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_POSITION, TRUE, TRUE);
@@ -1275,8 +1348,16 @@ LegendRectDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_container_add(GTK_CONTAINER(frame), table);
     gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+#else
     vbox = gtk_vbox_new(FALSE, 0);
+#endif
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
 
@@ -1293,7 +1374,11 @@ LegendRectDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_container_add(GTK_CONTAINER(frame), table);
     gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     fill_color_setup(d, table, i++);
@@ -1345,9 +1430,17 @@ LegendArcDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     gtk_dialog_add_button(GTK_DIALOG(wi), GTK_STOCK_DELETE, IDDELETE);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
+#endif
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_POSITION, TRUE, TRUE);
@@ -1371,7 +1464,11 @@ LegendArcDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_direction_entry();
@@ -1386,11 +1483,19 @@ LegendArcDialogSetup(GtkWidget *wi, void *data, int makewidget)
     add_widget_to_table(table, w, NULL, FALSE, i++);
     d->pieslice = w;
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 0);
+#else
     vbox = gtk_vbox_new(FALSE, 0);
+#endif
     gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 0);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = gtk_check_button_new_with_mnemonic(_("_Close path"));
@@ -1413,7 +1518,11 @@ LegendArcDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     fill_color_setup(d, table, i++);
@@ -1475,9 +1584,17 @@ LegendMarkDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     gtk_dialog_add_button(GTK_DIALOG(wi), GTK_STOCK_DELETE, IDDELETE);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
+#endif
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_POSITION, TRUE, TRUE);
@@ -1493,7 +1610,11 @@ LegendMarkDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     style_setup(d, table, i++);
@@ -1558,7 +1679,11 @@ legend_dialog_setup_sub(struct LegendDialog *d, GtkWidget *table, int i)
   add_widget_to_table(table, w, _("_Font:"), FALSE, i++);
   d->font = w;
 
+#if GTK_CHECK_VERSION(3, 2, 0)
+  btn_box = gtk_button_box_new(GTK_ORIENTATION_HORIZONTAL);
+#else
   btn_box = gtk_hbutton_box_new();
+#endif
   gtk_box_set_spacing(GTK_BOX(btn_box), 10);
   w = gtk_check_button_new_with_label("gtk-bold");
   gtk_button_set_use_stock(GTK_BUTTON(w), TRUE);
@@ -1709,9 +1834,17 @@ LegendTextDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     gtk_dialog_add_button(GTK_DIALOG(wi), GTK_STOCK_DELETE, IDDELETE);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
+#endif
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_POSITION, TRUE, TRUE);
@@ -1727,7 +1860,11 @@ LegendTextDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox), frame, FALSE, FALSE, 0);
 
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_text_entry(FALSE, TRUE);
@@ -1772,7 +1909,11 @@ LegendTextDefDialogSetup(GtkWidget *w, void *data, int makewidget)
   if (makewidget) {
     init_legend_dialog_widget_member(d);
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
     legend_dialog_setup_sub(d, table, 0);
 
     frame = gtk_frame_new(NULL);
@@ -2197,23 +2338,6 @@ LegendWinLegendUpdate(void *data, struct objlist *obj, int id, int sub_id)
     LegendWinUpdate(FALSE);
 }
 
-void
-LegendWinUpdate(int clear)
-{
-  int i;
-  struct LegendWin *d;
-
-  d = &(NgraphApp.LegendWin);
-  for (i = 0; i < LEGENDNUM; i++)
-    d->legend[i] = chkobjlastinst(d->obj[i]);
-
-  LegendWinExpose(NULL, NULL, NULL);
-
-  if (! clear && d->select >= 0 && d->legend_type >= 0) {
-    tree_store_select_nth(GTK_WIDGET(d->text), d->legend_type, d->select);
-  }
-}
-
 static int
 legend_list_must_rebuild(struct LegendWin *d)
 {
@@ -2240,6 +2364,33 @@ legend_list_must_rebuild(struct LegendWin *d)
   return r;
 }
 
+void
+LegendWinUpdate(int clear)
+{
+  int i;
+  struct LegendWin *d;
+
+  d = &(NgraphApp.LegendWin);
+  if (Menulock || Globallock)
+    return;
+
+  for (i = 0; i < LEGENDNUM; i++)
+    d->legend[i] = chkobjlastinst(d->obj[i]);
+
+  if (GTK_WIDGET(d->text) == NULL)
+    return;
+
+  if (legend_list_must_rebuild(d)) {
+    legend_list_build(d);
+  } else {
+    legend_list_set(d);
+  }
+
+  if (! clear && d->select >= 0 && d->legend_type >= 0) {
+    tree_store_select_nth(GTK_WIDGET(d->text), d->legend_type, d->select);
+  }
+}
+
 static void
 get_points(char *buf, int len, struct objlist *obj, int id, int *x, int *y, int style, char *ex)
 {
@@ -2491,31 +2642,6 @@ legend_list_set(struct LegendWin *d)
   }
 }
 
-static gboolean
-#if GTK_CHECK_VERSION(3, 0, 0)
-LegendWinExpose(GtkWidget *wi, cairo_t *cr, gpointer client_data)
-#else
-LegendWinExpose(GtkWidget *wi, GdkEvent *event, gpointer client_data)
-#endif
-{
-  struct LegendWin *d;
-
-  if (Menulock || Globallock)
-    return FALSE;
-
-  d = &(NgraphApp.LegendWin);
-  if (GTK_WIDGET(d->text) == NULL)
-    return FALSE;
-
-  if (legend_list_must_rebuild(d)) {
-    legend_list_build(d);
-  } else {
-    legend_list_set(d);
-  }
-
-  return FALSE;
-}
-
 static void
 popup_show_cb(GtkWidget *widget, gpointer user_data)
 {
@@ -2761,12 +2887,6 @@ CmLegendWindow(GtkToggleAction *action, gpointer client_data)
 
   dlg = tree_sub_window_create(d, "Legend Window", LEGEND_WIN_COL_NUM, Llist, Legendwin_xpm, Legendwin48_xpm);
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-  g_signal_connect(dlg, "draw", G_CALLBACK(LegendWinExpose), NULL);
-#else
-  g_signal_connect(dlg, "expose-event", G_CALLBACK(LegendWinExpose), NULL);
-#endif
-
   for (i = 0; i < LEGENDNUM; i++) {
     d->obj[i] = chkobject(legendlist[i]);
     d->legend[i] = chkobjlastinst(d->obj[i]);
@@ -2786,4 +2906,6 @@ CmLegendWindow(GtkToggleAction *action, gpointer client_data)
 
   sub_window_show_all((struct SubWin *) d);
   sub_window_set_geometry((struct SubWin *)d, TRUE);
+
+  LegendWinUpdate(TRUE);
 }
