@@ -63,11 +63,6 @@ static n_list_store Mlist[] = {
 #define MERG_WIN_COL_ID     1
 #define MERG_WIN_COL_FILE   2
 
-#if GTK_CHECK_VERSION(3, 0, 0)
-static gboolean MergeWinExpose(GtkWidget *wi, cairo_t *cr, gpointer client_data);
-#else
-static gboolean MergeWinExpose(GtkWidget *w, GdkEvent *event, gpointer client_data);
-#endif
 static void merge_list_set_val(struct SubWin *d, GtkTreeIter *iter, int row);
 
 static struct subwin_popup_list Popup_list[] = {
@@ -133,7 +128,11 @@ MergeDialogSetup(GtkWidget *wi, void *data, int makewidget)
   gtk_window_set_title(GTK_WINDOW(wi), title);
 
   if (makewidget) {
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_file_entry(d->Obj);
@@ -303,8 +302,14 @@ MergeWinUpdate(int clear)
   struct SubWin *d;
 
   d = &(NgraphApp.MergeWin);
+  if (GTK_WIDGET(d->text) == NULL)
+    return;
 
-  MergeWinExpose(NULL, NULL, NULL);
+  if (list_sub_window_must_rebuild(d)) {
+    list_sub_window_build(d, merge_list_set_val);
+  } else {
+    list_sub_window_set(d, merge_list_set_val);
+  }
 
   if (! clear && d->select >= 0) {
     list_store_select_int(GTK_WIDGET(d->text), MERG_WIN_COL_ID, d->select);
@@ -347,28 +352,6 @@ merge_list_set_val(struct SubWin *d, GtkTreeIter *iter, int row)
   }
 }
 
-static gboolean
-#if GTK_CHECK_VERSION(3, 0, 0)
-MergeWinExpose(GtkWidget *wi, cairo_t *cr, gpointer client_data)
-#else
-MergeWinExpose(GtkWidget *w, GdkEvent *event, gpointer client_data)
-#endif
-{
-  struct SubWin *d;
-
-  d = &(NgraphApp.MergeWin);
-  if (GTK_WIDGET(d->text) == NULL)
-    return FALSE;
-
-  if (list_sub_window_must_rebuild(d)) {
-    list_sub_window_build(d, merge_list_set_val);
-  } else {
-    list_sub_window_set(d, merge_list_set_val);
-  }
-
-  return FALSE;
-}
-
 static void
 popup_show_cb(GtkWidget *widget, gpointer user_data)
 {
@@ -437,7 +420,6 @@ CmMergeWindow(GtkToggleAction *action, gpointer client_data)
 {
   struct SubWin *d;
   int state;
-  GtkWidget *dlg;
 
   d = &(NgraphApp.MergeWin);
 
@@ -460,13 +442,7 @@ CmMergeWindow(GtkToggleAction *action, gpointer client_data)
   d->setup_dialog = MergeDialog;
   d->dialog = &DlgMerge;
 
-  dlg = list_sub_window_create(d, "Merge Window", MERG_WIN_COL_NUM, Mlist, Mergewin_xpm, Mergewin48_xpm);
-
-#if GTK_CHECK_VERSION(3, 0, 0)
-  g_signal_connect(dlg, "draw", G_CALLBACK(MergeWinExpose), NULL);
-#else
-  g_signal_connect(dlg, "expose-event", G_CALLBACK(MergeWinExpose), NULL);
-#endif
+  list_sub_window_create(d, "Merge Window", MERG_WIN_COL_NUM, Mlist, Mergewin_xpm, Mergewin48_xpm);
 
   d->obj = chkobject("merge");
   d->num = chkobjlastinst(d->obj);
@@ -481,4 +457,6 @@ CmMergeWindow(GtkToggleAction *action, gpointer client_data)
 
   sub_window_show_all(d);
   sub_window_set_geometry(d, TRUE);
+
+  MergeWinUpdate(TRUE);
 }
