@@ -110,32 +110,9 @@ DefaultDialogSetup(GtkWidget *wi, void *data, int makewidget)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->misc), FALSE);
 }
 
-static void
-add_str_with_int_to_array(struct narray *conf, char *str, int val)
-{
-  char *buf;
-
-  buf = (char *) g_malloc(BUF_SIZE);
-  if (buf) {
-    snprintf(buf, BUF_SIZE, "%s=%d", str, val);
-    arrayadd(conf, &buf);
-  }
-}
-
-static void
-save_ext_viewer_config(struct narray *conf)
-{
-  add_str_with_int_to_array(conf, "win_dpi", Menulocal.exwindpi);
-  add_str_with_int_to_array(conf, "win_width", Menulocal.exwinwidth);
-  add_str_with_int_to_array(conf, "win_height", Menulocal.exwinheight);
-  add_str_with_int_to_array(conf, "use_external_viewer", Menulocal.exwin_use_external);
-}
-
 static int
 save_config(int type)
 {
-  struct narray conf;
-
   if (!CheckIniFile()) {
     return 1;
   }
@@ -144,13 +121,6 @@ save_config(int type)
     menu_save_config(type);
   }
 
-  if (type & SAVE_CONFIG_TYPE_EXTERNAL_VIEWER) {
-    arrayinit(&conf, sizeof(char *));
-    save_ext_viewer_config(&conf);
-    replaceconfig("[gra2gtk]", &conf);
-    arraydel2(&conf);
-  }
-
   if (type & SAVE_CONFIG_TYPE_FONTS) {
     gra2cairo_save_config();
   }
@@ -218,13 +188,93 @@ DefaultDialog(struct DefaultDialog *data)
   data->CloseWindow = DefaultDialogClose;
 }
 
+static void
+active_script_changed(GtkComboBox *widget, gpointer user_data)
+{
+  struct SetScriptDialog *d;
+  struct script *addin;
+  int i, n;
+
+  d = (struct SetScriptDialog *) user_data;
+
+  n = gtk_combo_box_get_active(widget);
+  if (n < 1) {
+    gtk_entry_set_text(GTK_ENTRY(d->name), "");
+    gtk_entry_set_text(GTK_ENTRY(d->script), "");
+    gtk_entry_set_text(GTK_ENTRY(d->option), "");
+    gtk_entry_set_text(GTK_ENTRY(d->description), "");
+
+    return;
+  }
+
+  addin = Menulocal.addin_list;
+  for (i = 0; i < n - 1; i++) {
+    if (addin == NULL) {
+      return;
+    }
+    addin = addin->next;
+  }
+
+  gtk_entry_set_text(GTK_ENTRY(d->name), CHK_STR(addin->name));
+  gtk_entry_set_text(GTK_ENTRY(d->script), CHK_STR(addin->script));
+  gtk_entry_set_text(GTK_ENTRY(d->option), CHK_STR(addin->option));
+  gtk_entry_set_text(GTK_ENTRY(d->description), CHK_STR(addin->description));
+
+  return;
+}
+
+static void
+remove_char(char *str, int c)
+{
+  int i, j, n;
+
+  if (str == NULL) {
+    return;
+  }
+
+  n = strlen(str);
+  for (i = 0; i < n; i++) {
+    if (str[i] == c) {
+      for (j = i; j < n; j++) {
+	str[j] = str[j + 1];
+      }
+      n--;
+    }
+  }
+}
+
 static void
 SetScriptDialogSetupItem(GtkWidget *w, struct SetScriptDialog *d)
 {
-  gtk_entry_set_text(GTK_ENTRY(d->name), CHK_STR(d->Script->name));
-  gtk_entry_set_text(GTK_ENTRY(d->script), CHK_STR(d->Script->script));
-  gtk_entry_set_text(GTK_ENTRY(d->option), CHK_STR(d->Script->option));
-  gtk_entry_set_text(GTK_ENTRY(d->description), CHK_STR(d->Script->description));
+  struct script *addin;
+  char *title;
+
+  combo_box_clear(d->addins);
+  combo_box_append_text(d->addins, "Custom");
+  if (Menulocal.addin_list == NULL) {
+    set_widget_sensitivity_with_label(d->addins, FALSE);
+  } else {
+    set_widget_sensitivity_with_label(d->addins, TRUE);
+    for (addin = Menulocal.addin_list; addin; addin = addin->next) {
+      title = g_strdup(addin->name);
+      if (title) {
+	remove_char(title, '_');
+	combo_box_append_text(d->addins, title);
+	g_free(title);
+      }
+    }
+    active_script_changed(GTK_COMBO_BOX(d->addins), d);
+  }
+
+  if (d->Script->name) {
+    combo_box_set_active(d->addins, 0);
+    gtk_entry_set_text(GTK_ENTRY(d->name), CHK_STR(d->Script->name));
+    gtk_entry_set_text(GTK_ENTRY(d->script), CHK_STR(d->Script->script));
+    gtk_entry_set_text(GTK_ENTRY(d->option), CHK_STR(d->Script->option));
+    gtk_entry_set_text(GTK_ENTRY(d->description), CHK_STR(d->Script->description));
+  } else {
+    combo_box_set_active(d->addins, 1);
+  }
 }
 
 #if USE_ENTRY_ICON
@@ -252,9 +302,19 @@ SetScriptDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
   d = (struct SetScriptDialog *) data;
   if (makewidget) {
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
+
+    w = combo_box_create();
+    add_widget_to_table(table, w, _("_Add-in:"), TRUE, i++);
+    g_signal_connect(w, "changed", G_CALLBACK(active_script_changed), d);
+    d->addins = w;
+
     w = create_text_entry(FALSE, TRUE);
     add_widget_to_table(table, w, _("_Name:"), TRUE, i++);
     d->name = w;
@@ -486,7 +546,11 @@ SetDriverDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
   d = (struct SetDriverDialog *) data;
   if (makewidget) {
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_text_entry(FALSE, TRUE);
@@ -848,7 +912,11 @@ FontSettingDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     int j;
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     j = 0;
     w = gtk_entry_new();
@@ -864,7 +932,11 @@ FontSettingDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(d->vbox), table, FALSE, FALSE, 4);
 
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
+#endif
 
     swin = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(swin), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
@@ -878,7 +950,11 @@ FontSettingDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     gtk_box_pack_start(GTK_BOX(hbox), swin, TRUE, TRUE, 4);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
 
     w = gtk_button_new_from_stock(GTK_STOCK_ADD);
     g_signal_connect(w, "clicked", G_CALLBACK(FontSettingDialogAddAlternative), d);
@@ -1112,7 +1188,11 @@ PrefFontDialogSetup(GtkWidget *wi, void *data, int makewidget)
   if (makewidget) {
     gtk_dialog_add_button(GTK_DIALOG(wi), GTK_STOCK_SAVE, IDSAVE);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox = gtk_vbox_new(FALSE, 4);
+#endif
     PrefFontDialogCreateWidgets(d, vbox, sizeof(list) / sizeof(*list), list);
     gtk_window_set_default_size(GTK_WINDOW(wi), 550, 300);
   }
@@ -1215,11 +1295,20 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_dialog_add_button(GTK_DIALOG(wi), GTK_STOCK_CANCEL, IDCANCEL);
     gtk_dialog_add_button(GTK_DIALOG(wi), GTK_STOCK_SAVE, IDSAVE);
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox2 = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+    vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     hbox2 = gtk_hbox_new(FALSE, 4);
     vbox2 = gtk_vbox_new(FALSE, 4);
+#endif
 
     frame = gtk_frame_new(_("External programs"));
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
 
@@ -1241,7 +1330,11 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     frame = gtk_frame_new(NULL);
     gtk_frame_set_label(GTK_FRAME(frame), _("Save graph"));
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = combo_box_create();
@@ -1265,7 +1358,11 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
     frame = gtk_frame_new(NULL);
     gtk_frame_set_label(GTK_FRAME(frame), _("Load graph"));
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = gtk_check_button_new_with_mnemonic(_("_Expand include file"));
@@ -1286,10 +1383,18 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(hbox2), vbox2, TRUE, TRUE, 4);
 
 
+#if GTK_CHECK_VERSION(3, 0, 0)
+    vbox2 = gtk_box_new(GTK_ORIENTATION_VERTICAL, 4);
+#else
     vbox2 = gtk_vbox_new(FALSE, 4);
+#endif
 
     frame = gtk_frame_new(_("Size"));
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = create_spin_entry(1, HIST_SIZE_MAX, 1, FALSE, TRUE);
@@ -1309,7 +1414,11 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
 
 
     frame = gtk_frame_new(_("Font"));
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = gtk_font_button_new();
@@ -1328,7 +1437,11 @@ MiscDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_box_pack_start(GTK_BOX(vbox2), frame, FALSE, FALSE, 4);
 
     frame = gtk_frame_new(_("Miscellaneous"));
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = gtk_check_button_new_with_mnemonic(_("_Check \"change current directory\""));
@@ -1517,7 +1630,11 @@ ExViewerDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_dialog_add_button(GTK_DIALOG(wi), GTK_STOCK_CANCEL, IDCANCEL);
     gtk_dialog_add_button(GTK_DIALOG(wi), GTK_STOCK_SAVE, IDSAVE);
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i  = 0;
     w = gtk_check_button_new_with_mnemonic(_("use _External previewer"));
@@ -1525,7 +1642,11 @@ ExViewerDialogSetup(GtkWidget *wi, void *data, int makewidget)
     g_signal_connect(w, "toggled", G_CALLBACK(use_external_toggled), d);
     d->use_external = w;
 
+#if GTK_CHECK_VERSION(3, 2, 0)
+    w = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, 20, 620, 1);
+#else
     w = gtk_hscale_new_with_range(20, 620, 1);
+#endif
     d->dpi = w;
     add_widget_to_table(table, w, "_DPI:", TRUE, i++);
 
@@ -1574,7 +1695,11 @@ static void
 ViewerDialogSetupItem(GtkWidget *w, struct ViewerDialog *d)
 {
   int a;
+#if GTK_CHECK_VERSION(3, 4, 0)
+  GdkRGBA color;
+#else
   GdkColor color;
+#endif
 
   getobj(d->Obj, "dpi", d->Id, 0, NULL, &(d->dpis));
   gtk_range_set_value(GTK_RANGE(d->dpi), d->dpis);
@@ -1592,10 +1717,20 @@ ViewerDialogSetupItem(GtkWidget *w, struct ViewerDialog *d)
 
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->preserve_width), Menulocal.preserve_width);
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+  color.red = Menulocal.bg_r;
+  color.green = Menulocal.bg_g;
+  color.blue = Menulocal.bg_b;
+  color.alpha = 1;
+  gtk_color_chooser_set_use_alpha(GTK_COLOR_CHOOSER(d->bgcol), FALSE);
+  gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(d->bgcol), &color);
+#else
   color.red = Menulocal.bg_r * 65535;
   color.green = Menulocal.bg_g * 65535;
   color.blue = Menulocal.bg_b * 65535;
+  gtk_color_button_set_use_alpha(GTK_COLOR_BUTTON(d->bgcol), FALSE);
   gtk_color_button_set_color(GTK_COLOR_BUTTON(d->bgcol), &color);
+#endif
 }
 
 static void
@@ -1614,7 +1749,7 @@ load_file_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 static void
 ViewerDialogSetup(GtkWidget *wi, void *data, int makewidget)
 {
-  GtkWidget *w, *hbox, *table;
+  GtkWidget *w, *table;
   struct ViewerDialog *d;
   int i, j;
 
@@ -1624,14 +1759,20 @@ ViewerDialogSetup(GtkWidget *wi, void *data, int makewidget)
     gtk_dialog_add_button(GTK_DIALOG(wi), GTK_STOCK_CANCEL, IDCANCEL);
     gtk_dialog_add_button(GTK_DIALOG(wi), GTK_STOCK_SAVE, IDSAVE);
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
-    hbox = gtk_hbox_new(FALSE, 4);
+#if GTK_CHECK_VERSION(3, 2, 0)
+    w = gtk_scale_new_with_range(GTK_ORIENTATION_HORIZONTAL, 20, 620, 1);
+#else
     w = gtk_hscale_new_with_range(20, 620, 1);
+#endif
     d->dpi = w;
-    item_setup(hbox, w, "_DPI:", TRUE);
-    add_widget_to_table(table, hbox, NULL, TRUE, i++);
+    add_widget_to_table(table, w, "_DPI:", TRUE, i++);
 
     w = create_spin_entry_type(SPIN_BUTTON_TYPE_LENGTH, FALSE, TRUE);
     spin_entry_set_range(w, 1, GRID_MAX);
@@ -1678,7 +1819,11 @@ ViewerDialogClose(GtkWidget *w, void *data)
 {
   struct ViewerDialog *d;
   int ret, dpi, a;
+#if GTK_CHECK_VERSION(3, 4, 0)
+  GdkRGBA color;
+#else
   GdkColor color;
+#endif
 
   d = (struct ViewerDialog *) data;
 
@@ -1712,10 +1857,17 @@ ViewerDialogClose(GtkWidget *w, void *data)
   Menulocal.preserve_width =
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(d->preserve_width));
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+  gtk_color_chooser_get_rgba(GTK_COLOR_CHOOSER(d->bgcol), &color);
+  Menulocal.bg_r = color.red;
+  Menulocal.bg_g = color.green;
+  Menulocal.bg_b = color.blue;
+#else
   gtk_color_button_get_color(GTK_COLOR_BUTTON(d->bgcol), &color);
   Menulocal.bg_r = color.red / 65535.0;
   Menulocal.bg_g = color.green / 65535.0;
   Menulocal.bg_b = color.blue / 65535.0;
+#endif
 
   Menulocal.grid = spin_entry_get_val(d->grid);
 
