@@ -149,7 +149,11 @@ DriverDialogSetup(GtkWidget *wi, void *data, int makewidget)
   d = (struct DriverDialog *) data;
   if (makewidget) {
 
+#if GTK_CHECK_VERSION(3, 4, 0)
+    table = gtk_grid_new();
+#else
     table = gtk_table_new(1, 2, FALSE);
+#endif
 
     i = 0;
     w = combo_box_create();
@@ -250,7 +254,11 @@ OutputDataDialogSetup(GtkWidget *wi, void *data, int makewidget)
   if (makewidget) {
     w = create_spin_entry(0, 200, 1, FALSE, TRUE);
     d->div_entry = w;
+#if GTK_CHECK_VERSION(3, 0, 0)
+    hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 4);
+#else
     hbox = gtk_hbox_new(FALSE, 4);
+#endif
     item_setup(hbox, w, _("_Div:"), TRUE);
     gtk_box_pack_start(GTK_BOX(d->vbox), hbox, FALSE, FALSE, 4);
   }
@@ -583,17 +591,6 @@ init_graobj(struct objlist *graobj, int id, char *dev_name, int dev_oid)
   putobj(graobj, "device", id, device);
 }
 
-static void
-init_print(GtkPrintOperation *operation, GtkPrintContext *context, gpointer user_data)
-{
-  GtkPrintSettings *settings;
-  int n;
-
-  settings = gtk_print_operation_get_print_settings(operation);
-  n = gtk_print_settings_get_n_copies(settings);
-  gtk_print_operation_set_n_pages(operation, n);
-}
-
 void
 CmOutputPrinter(int select_file, int show_dialog)
 {
@@ -674,7 +671,6 @@ CmOutputPrinter(int select_file, int show_dialog)
   pobj.g2wobj = g2wobj;
   pobj.g2winst = g2winst;
   g_signal_connect(print, "draw_page", G_CALLBACK(draw_page), &pobj);
-  g_signal_connect(print, "begin_print", G_CALLBACK(init_print), NULL);
 
   switch (show_dialog) {
   case PRINT_SHOW_DIALOG_NONE:
@@ -800,6 +796,8 @@ CmOutputViewerB(GtkAction *wi, gpointer client_data)
     g2winst = chkobjinst(g2wobj, g2wid);
     _getobj(g2wobj, "oid", g2winst, &g2woid);
     putobj(g2wobj, "dpi", g2wid, &(Menulocal.exwindpi));
+    putobj(g2wobj, "width", g2wid, &Menulocal.exwinwidth);
+    putobj(g2wobj, "height", g2wid, &Menulocal.exwinheight);
     c = Menulocal.bg_r * 255.0;
     putobj(g2wobj, "BR", g2wid, &c);
     c = Menulocal.bg_g * 255.0;
