@@ -13,6 +13,7 @@
 
 #ifdef HAVE_LIBGSL
 #include <gsl/gsl_sf.h>
+#include <gsl/gsl_rng.h>
 #endif
 
 #include "ntime.h"
@@ -919,7 +920,8 @@ math_func_rand(MathFunctionCallExpression *exp, MathEquation *eq, MathValue *rva
 {
   MATH_CHECK_ARG(rval, exp->buf[0]);
 
-  rval->val =  (rand() / ((double) RAND_MAX + 1)) * exp->buf[0].val.val;
+  rval->val = g_random_double() * exp->buf[0].val.val;
+
   return 0;
 }
 
@@ -928,7 +930,7 @@ math_func_srand(MathFunctionCallExpression *exp, MathEquation *eq, MathValue *rv
 {
   MATH_CHECK_ARG(rval, exp->buf[0]);
 
-  srand(exp->buf[0].val.val);
+  g_random_set_seed(exp->buf[0].val.val);
   rval->val = exp->buf[0].val.val;
 
   return 0;
@@ -2167,6 +2169,33 @@ math_func_jl(MathFunctionCallExpression *exp, MathEquation *eq, MathValue *rval)
   }
   return r;
 }
+
+int
+math_func_choose(MathFunctionCallExpression *exp, MathEquation *eq, MathValue *rval)
+{
+  unsigned int m, n;
+  int r;
+  gsl_sf_result val;
+
+  MATH_CHECK_ARG(rval, exp->buf[0]);
+  MATH_CHECK_ARG(rval, exp->buf[1]);
+
+  if (exp->buf[0].val.val < 0 || exp->buf[1].val.val < 0) {
+    printf("math_func_choose\n");
+    rval->type = MATH_VALUE_ERROR;
+    return 1;
+  }
+
+  n = exp->buf[0].val.val;
+  m = exp->buf[1].val.val;
+
+  r = gsl_sf_choose_e(n, m, &val);
+  rval->val = val.val;
+  if (r) {
+    rval->type = MATH_VALUE_ERROR;
+  }
+  return r;
+}
 #endif
 
 int
