@@ -624,184 +624,186 @@ math_expression_free(MathExpression *exp)
 
 }
 
+static double Fact_data[] = {
+  1.000000000000000e+00,
+  1.000000000000000e+00,
+  2.000000000000000e+00,
+  6.000000000000000e+00,
+  2.400000000000000e+01,
+  1.200000000000000e+02,
+  7.200000000000000e+02,
+  5.040000000000000e+03,
+  4.032000000000000e+04,
+  3.628800000000000e+05,
+  3.628800000000000e+06,
+  3.991680000000000e+07,
+  4.790016000000000e+08,
+  6.227020800000000e+09,
+  8.717829120000000e+10,
+  1.307674368000000e+12,
+  2.092278988800000e+13,
+  3.556874280960000e+14,
+  6.402373705728000e+15,
+  1.216451004088320e+17,
+  2.432902008176640e+18,
+  5.109094217170944e+19,
+  1.124000727777608e+21,
+  2.585201673888498e+22,
+  6.204484017332394e+23,
+  1.551121004333099e+25,
+  4.032914611266057e+26,
+  1.088886945041835e+28,
+  3.048883446117139e+29,
+  8.841761993739702e+30,
+  2.652528598121911e+32,
+  8.222838654177922e+33,
+  2.631308369336935e+35,
+  8.683317618811886e+36,
+  2.952327990396042e+38,
+  1.033314796638615e+40,
+  3.719933267899013e+41,
+  1.376375309122635e+43,
+  5.230226174666011e+44,
+  2.039788208119744e+46,
+  8.159152832478977e+47,
+  3.345252661316381e+49,
+  1.405006117752880e+51,
+  6.041526306337383e+52,
+  2.658271574788449e+54,
+  1.196222208654802e+56,
+  5.502622159812089e+57,
+  2.586232415111682e+59,
+  1.241391559253607e+61,
+  6.082818640342675e+62,
+  3.041409320171338e+64,
+  1.551118753287382e+66,
+  8.065817517094388e+67,
+  4.274883284060025e+69,
+  2.308436973392414e+71,
+  1.269640335365828e+73,
+  7.109985878048635e+74,
+  4.052691950487721e+76,
+  2.350561331282878e+78,
+  1.386831185456898e+80,
+  8.320987112741390e+81,
+  5.075802138772248e+83,
+  3.146997326038794e+85,
+  1.982608315404440e+87,
+  1.268869321858842e+89,
+  8.247650592082472e+90,
+  5.443449390774431e+92,
+  3.647111091818868e+94,
+  2.480035542436831e+96,
+  1.711224524281413e+98,
+  1.197857166996989e+100,
+  8.504785885678623e+101,
+  6.123445837688608e+103,
+  4.470115461512684e+105,
+  3.307885441519386e+107,
+  2.480914081139540e+109,
+  1.885494701666050e+111,
+  1.451830920282859e+113,
+  1.132428117820630e+115,
+  8.946182130782976e+116,
+  7.156945704626381e+118,
+  5.797126020747368e+120,
+  4.753643337012842e+122,
+  3.945523969720659e+124,
+  3.314240134565353e+126,
+  2.817104114380550e+128,
+  2.422709538367273e+130,
+  2.107757298379528e+132,
+  1.854826422573984e+134,
+  1.650795516090846e+136,
+  1.485715964481762e+138,
+  1.352001527678403e+140,
+  1.243841405464131e+142,
+  1.156772507081642e+144,
+  1.087366156656743e+146,
+  1.032997848823906e+148,
+  9.916779348709496e+149,
+  9.619275968248212e+151,
+  9.426890448883248e+153,
+  9.332621544394415e+155,
+  9.332621544394415e+157,
+  9.425947759838360e+159,
+  9.614466715035127e+161,
+  9.902900716486180e+163,
+  1.029901674514563e+166,
+  1.081396758240291e+168,
+  1.146280563734708e+170,
+  1.226520203196138e+172,
+  1.324641819451829e+174,
+  1.443859583202494e+176,
+  1.588245541522743e+178,
+  1.762952551090245e+180,
+  1.974506857221074e+182,
+  2.231192748659814e+184,
+  2.543559733472188e+186,
+  2.925093693493016e+188,
+  3.393108684451898e+190,
+  3.969937160808721e+192,
+  4.684525849754291e+194,
+  5.574585761207606e+196,
+  6.689502913449127e+198,
+  8.094298525273444e+200,
+  9.875044200833601e+202,
+  1.214630436702533e+205,
+  1.506141741511141e+207,
+  1.882677176888926e+209,
+  2.372173242880047e+211,
+  3.012660018457659e+213,
+  3.856204823625804e+215,
+  4.974504222477287e+217,
+  6.466855489220474e+219,
+  8.471580690878821e+221,
+  1.118248651196004e+224,
+  1.487270706090686e+226,
+  1.992942746161519e+228,
+  2.690472707318050e+230,
+  3.659042881952549e+232,
+  5.012888748274992e+234,
+  6.917786472619489e+236,
+  9.615723196941089e+238,
+  1.346201247571753e+241,
+  1.898143759076171e+243,
+  2.695364137888163e+245,
+  3.854370717180073e+247,
+  5.550293832739304e+249,
+  8.047926057471992e+251,
+  1.174997204390911e+254,
+  1.727245890454639e+256,
+  2.556323917872865e+258,
+  3.808922637630570e+260,
+  5.713383956445855e+262,
+  8.627209774233240e+264,
+  1.311335885683452e+267,
+  2.006343905095682e+269,
+  3.089769613847351e+271,
+  4.789142901463394e+273,
+  7.471062926282894e+275,
+  1.172956879426414e+278,
+  1.853271869493735e+280,
+  2.946702272495038e+282,
+  4.714723635992062e+284,
+  7.590705053947219e+286,
+  1.229694218739449e+289,
+  2.004401576545303e+291,
+  3.287218585534296e+293,
+  5.423910666131589e+295,
+  9.003691705778438e+297,
+  1.503616514864999e+300,
+  2.526075744973198e+302,
+  4.269068009004705e+304,
+};
+
+#define FACT_MAX (sizeof(Fact_data) / sizeof(*Fact_data))
+
 static double
 factorial(unsigned int n)
 {
-  static double data[] = {
-    1.000000000000000e+00,
-    1.000000000000000e+00,
-    2.000000000000000e+00,
-    6.000000000000000e+00,
-    2.400000000000000e+01,
-    1.200000000000000e+02,
-    7.200000000000000e+02,
-    5.040000000000000e+03,
-    4.032000000000000e+04,
-    3.628800000000000e+05,
-    3.628800000000000e+06,
-    3.991680000000000e+07,
-    4.790016000000000e+08,
-    6.227020800000000e+09,
-    8.717829120000000e+10,
-    1.307674368000000e+12,
-    2.092278988800000e+13,
-    3.556874280960000e+14,
-    6.402373705728000e+15,
-    1.216451004088320e+17,
-    2.432902008176640e+18,
-    5.109094217170944e+19,
-    1.124000727777608e+21,
-    2.585201673888498e+22,
-    6.204484017332394e+23,
-    1.551121004333099e+25,
-    4.032914611266057e+26,
-    1.088886945041835e+28,
-    3.048883446117139e+29,
-    8.841761993739702e+30,
-    2.652528598121911e+32,
-    8.222838654177922e+33,
-    2.631308369336935e+35,
-    8.683317618811886e+36,
-    2.952327990396042e+38,
-    1.033314796638615e+40,
-    3.719933267899013e+41,
-    1.376375309122635e+43,
-    5.230226174666011e+44,
-    2.039788208119744e+46,
-    8.159152832478977e+47,
-    3.345252661316381e+49,
-    1.405006117752880e+51,
-    6.041526306337383e+52,
-    2.658271574788449e+54,
-    1.196222208654802e+56,
-    5.502622159812089e+57,
-    2.586232415111682e+59,
-    1.241391559253607e+61,
-    6.082818640342675e+62,
-    3.041409320171338e+64,
-    1.551118753287382e+66,
-    8.065817517094388e+67,
-    4.274883284060025e+69,
-    2.308436973392414e+71,
-    1.269640335365828e+73,
-    7.109985878048635e+74,
-    4.052691950487721e+76,
-    2.350561331282878e+78,
-    1.386831185456898e+80,
-    8.320987112741390e+81,
-    5.075802138772248e+83,
-    3.146997326038794e+85,
-    1.982608315404440e+87,
-    1.268869321858842e+89,
-    8.247650592082472e+90,
-    5.443449390774431e+92,
-    3.647111091818868e+94,
-    2.480035542436831e+96,
-    1.711224524281413e+98,
-    1.197857166996989e+100,
-    8.504785885678623e+101,
-    6.123445837688608e+103,
-    4.470115461512684e+105,
-    3.307885441519386e+107,
-    2.480914081139540e+109,
-    1.885494701666050e+111,
-    1.451830920282859e+113,
-    1.132428117820630e+115,
-    8.946182130782976e+116,
-    7.156945704626381e+118,
-    5.797126020747368e+120,
-    4.753643337012842e+122,
-    3.945523969720659e+124,
-    3.314240134565353e+126,
-    2.817104114380550e+128,
-    2.422709538367273e+130,
-    2.107757298379528e+132,
-    1.854826422573984e+134,
-    1.650795516090846e+136,
-    1.485715964481762e+138,
-    1.352001527678403e+140,
-    1.243841405464131e+142,
-    1.156772507081642e+144,
-    1.087366156656743e+146,
-    1.032997848823906e+148,
-    9.916779348709496e+149,
-    9.619275968248212e+151,
-    9.426890448883248e+153,
-    9.332621544394415e+155,
-    9.332621544394415e+157,
-    9.425947759838360e+159,
-    9.614466715035127e+161,
-    9.902900716486180e+163,
-    1.029901674514563e+166,
-    1.081396758240291e+168,
-    1.146280563734708e+170,
-    1.226520203196138e+172,
-    1.324641819451829e+174,
-    1.443859583202494e+176,
-    1.588245541522743e+178,
-    1.762952551090245e+180,
-    1.974506857221074e+182,
-    2.231192748659814e+184,
-    2.543559733472188e+186,
-    2.925093693493016e+188,
-    3.393108684451898e+190,
-    3.969937160808721e+192,
-    4.684525849754291e+194,
-    5.574585761207606e+196,
-    6.689502913449127e+198,
-    8.094298525273444e+200,
-    9.875044200833601e+202,
-    1.214630436702533e+205,
-    1.506141741511141e+207,
-    1.882677176888926e+209,
-    2.372173242880047e+211,
-    3.012660018457659e+213,
-    3.856204823625804e+215,
-    4.974504222477287e+217,
-    6.466855489220474e+219,
-    8.471580690878821e+221,
-    1.118248651196004e+224,
-    1.487270706090686e+226,
-    1.992942746161519e+228,
-    2.690472707318050e+230,
-    3.659042881952549e+232,
-    5.012888748274992e+234,
-    6.917786472619489e+236,
-    9.615723196941089e+238,
-    1.346201247571753e+241,
-    1.898143759076171e+243,
-    2.695364137888163e+245,
-    3.854370717180073e+247,
-    5.550293832739304e+249,
-    8.047926057471992e+251,
-    1.174997204390911e+254,
-    1.727245890454639e+256,
-    2.556323917872865e+258,
-    3.808922637630570e+260,
-    5.713383956445855e+262,
-    8.627209774233240e+264,
-    1.311335885683452e+267,
-    2.006343905095682e+269,
-    3.089769613847351e+271,
-    4.789142901463394e+273,
-    7.471062926282894e+275,
-    1.172956879426414e+278,
-    1.853271869493735e+280,
-    2.946702272495038e+282,
-    4.714723635992062e+284,
-    7.590705053947219e+286,
-    1.229694218739449e+289,
-    2.004401576545303e+291,
-    3.287218585534296e+293,
-    5.423910666131589e+295,
-    9.003691705778438e+297,
-    1.503616514864999e+300,
-    2.526075744973198e+302,
-    4.269068009004705e+304,
-  };
-
-  if (n < (sizeof(data) / sizeof(*data))) {
-    return data[n];
+  if (n < FACT_MAX) {
+    return Fact_data[n];
   }
 
   return HUGE_VAL;
@@ -1521,7 +1523,6 @@ calc(MathExpression *exp, MathValue *val)
     MATH_CHECK_VAL(val, left);
     MATH_CHECK_VAL(val, right);
     if (right.val == 0) {
-      val->type = MATH_VALUE_ERROR;
       return 1;
     }
     val->val = fmod(left.val, right.val);
@@ -1595,8 +1596,7 @@ calc(MathExpression *exp, MathValue *val)
 
     MATH_CHECK_VAL(val, operand);
 
-    if (operand.val < 0 || operand.val > INT_MAX) {
-      val->type = MATH_VALUE_ERROR;
+    if (operand.val < 0 || operand.val >= FACT_MAX) {
       return 1;
     }
 
@@ -1623,12 +1623,10 @@ calc(MathExpression *exp, MathValue *val)
     }
     if (exp->u.assign.left->type == MATH_EXPRESSION_TYPE_VARIABLE) {
       if (math_equation_set_var(exp->equation, exp->u.assign.left->u.index, &right)) {
-	val->type = MATH_VALUE_ERROR;
 	return 1;
       }
     } else {
       if (set_val_to_array(exp->u.assign.left, &right, exp->u.assign.op)) {
-	val->type = MATH_VALUE_ERROR;
 	return 1;
       }
     }
@@ -1640,13 +1638,11 @@ calc(MathExpression *exp, MathValue *val)
       return 1;
     }
     if (math_equation_get_array_val(exp->equation, exp->u.array.index, operand.val, val)) {
-      val->type = MATH_VALUE_ERROR;
       return 1;
     }
     break;
   case MATH_EXPRESSION_TYPE_PRM:
     if (exp->u.prm.prm->data == NULL) {
-      val->type = MATH_VALUE_ERROR;
       return 1;
     }
     *val = exp->u.prm.prm->data[exp->u.prm.index];
@@ -1681,7 +1677,8 @@ math_expression_calculate(MathExpression *exp, MathValue *val)
     v.val = 0;
     v.type = MATH_VALUE_NORMAL;
     if (calc(ptr, &v)) {
-      *val = v;
+      val->val = v.val;
+      val->type = MATH_VALUE_ERROR;
       return 1;
     }
     ptr = ptr->next;
